from fabric.api import lcd, env, local, hosts, warn_only
import fabric.contrib.project as project
import multiprocessing
import os

env.use_ssh_config = True
# Local path configuration (can be absolute or relative to fabfile)
env.deploy_path = 'output'
DEPLOY_PATH = env.deploy_path

# Remote server configuration
production = 'www.shore.co.il'
dest_path = '/var/www/htdocs/www.shore.co.il/blog/'

UID = os.getuid()


def clean():
    """Remove generated files"""
    local(f'rm -rf __pycache__/ {DEPLOY_PATH}/* sitespeed-result/ lighthouse-result/')


def build():
    """Build local version of site"""
    local('pelican -s pelicanconf.py')


def regenerate():
    """Automatically regenerate site upon file modification"""
    local('pelican -D -r -s pelicanconf.py')


def serve():
    """Serve site at http://localhost:8080/"""
    with lcd(DEPLOY_PATH):
        local('python -m http.server 8080')


def dev():
    """Auto-regenerate files and serve at http://localhost:8080/"""
    server_process = multiprocessing.Process(target=serve, daemon=True)
    server_process.start()
    regenerate()
    server_process.terminate()
    server_process.join(timeout=3)


def preview():
    """Build production version of site"""
    local('pelican -s publishconf.py')


@hosts(production)
def publish():
    """Publish to production via rsync"""
    preview()
    project.rsync_project(
        remote_dir=dest_path,
        exclude=".DS_Store",
        local_dir=DEPLOY_PATH.rstrip('/') + '/',
        delete=True,
        extra_opts='-c',
    )


def sitespeed():
    """Run sitespeed test against the local dev server"""
    build()
    with warn_only():
        local('docker run --rm --privileged --net=host gliderlabs/hostlocal')
    server_process = multiprocessing.Process(target=serve, daemon=True)
    server_process.start()
    local(f'docker run --rm --shm-size=1g -u {UID} -v "$PWD:/sitespeed.io" sitespeedio/sitespeed.io http://169.254.255.254:8080/')
    server_process.terminate()
    server_process.join(timeout=3)


def lighthouse():
    """Run Chrome's Lighthouse report against the local dev server"""
    build()
    with warn_only():
        local('docker run --rm --privileged --net=host gliderlabs/hostlocal')
    server_process = multiprocessing.Process(target=serve, daemon=True)
    server_process.start()
    local('mkdir -p lighthouse-result')
    local('docker run --rm -u {UID} -v "$PWD/lighthouse-result:/home/chrome/reports" --cap-add=SYS_ADMIN --user=1000 justinribeiro/lighthouse lighthouse --chrome-flags="--headless --no-sandbox --disable-gpu" http://169.254.255.254:8080/')
    server_process.terminate()
    server_process.join(timeout=3)
